
#ifndef _DENSITY_MAP_H_
#define _DENSITY_MAP_H_

#include "Vector3.h"


#include <string>
#include <vector>
#include <stack>
#include <map>
using namespace std;


class Regions;
class Complex;


class DensityMap
{
public:

	string											name;

	vector< vector< vector< double   > > >			M, L;
	vector< vector< vector< Vector3d > > >			G;

	int												n[3], nstart[3];
	Vector3d										O, step, vSize;
	double											dMaxVal, dMinVal, dCellVolume, dAvgDensity;

	DensityMap ();

	void ReadText (const char *fname);
	void ReadCCP4 (const char *fname);
	void ReadMRC (const char *fname, bool bPrint=false);
	void WriteCCP4 (const char *fname);
	void WriteMRC (const char *fname);
	void WriteDF3 (const char *fname);

	void Allocate (int n1, int n2, int n3);
	void CopyHeaderFrom ( DensityMap *dm );
	void CopyFrom ( DensityMap *dm );
	void Add ( DensityMap *dm );
	void TAdd ( DensityMap *dm );
	DensityMap* SubSample ( int sn );

	void SliceX (int x, char *suffix);
	void GradSliceX (int x, char *suffix);
	double AverageBlockDensity ( int il, int ih, int jl, int jh, int kl, int kh );

	DensityMap* Smooth ();
	DensityMap* GaussianConvolve ( double dWidth );
	void GaussianConvolveTo ( double dWidth, DensityMap *dm );

	void ZeroBelow (double val);
	void DifToVal ( double dval );
	void Equalize ();
	void Normalize ();
	double EqualizeStep (bool bPrint=false);

	double DensityValue ( double *p );
	double AverageDensityValue ( Complex *c );

	DensityMap* ComputeGradients ( double exp_f = -1.0 );
	void GradientVal ( Vector3d &p, Vector3d &g );
	void InterpolateGradients ( Complex *c, vector<Vector3d> &grads );
	
	void ComputeLaplacian ();
	void LaplacianSmooth ( double step_size );

	Regions* Watershed ( double dThr=-1e99, double dMaxVol=1e99 );
	double RegionsThreshold ( Regions *regs );
};



#endif //_VTK_EMSEG_H_

