

#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>


class Vector3d
{
public:
	double p[3];

	Vector3d () {
		p[0] = 0.0;
		p[1] = 0.0;
		p[2] = 0.0;
	}
	
	Vector3d ( double x_, double y_, double z_ )
	{
		p[0] = x_;
		p[1] = y_;
		p[2] = z_;
	}
	
	Vector3d (const Vector3d& v)
	{
		p[0] = v[0];
		p[1] = v[1];
		p[2] = v[2];
	}

	
	operator const double* () const {
		return p;
	}
	operator double* () {
		return p;
	}

	double& operator[] (int i)
	{
		return p[i];
	}
	
	Vector3d operator+ ( Vector3d v )
	{
		return Vector3d ( p[0] + v[0], p[1] + v[1], p[2] + v[2] );
	}

	void operator= ( Vector3d v )
	{
		p[0] = v[0];
		p[1] = v[1];
		p[2] = v[2];
	}

	void operator+= ( Vector3d v )
	{
		p[0] += v[0]; p[1] += v[1]; p[2] += v[2];
	}

	void operator/= ( double v )
	{
		p[0] /= v; p[1] /= v; p[2] /= v;
	}

	Vector3d operator- ( Vector3d v )
	{
		return Vector3d ( p[0] - v[0], p[1] - v[1], p[2] - v[2] );
	}

	Vector3d operator* ( double v )
	{
		return Vector3d ( p[0]*v, p[1]*v, p[2]*v );
	}
	
	double Length ()
	{
		return sqrt ( p[0]*p[0] + p[1]*p[1] + p[2]*p[2] );
	}
	
	double SquaredLength ()
	{
		return  p[0]*p[0] + p[1]*p[1] + p[2]*p[2];
	}
};


class Vector3f
	{
	public:
		float p[3];
		
		Vector3f () {
			p[0] = 0.0;
			p[1] = 0.0;
			p[2] = 0.0;
		}
		
		Vector3f ( float x_, float y_, float z_ )
		{
			p[0] = x_;
			p[1] = y_;
			p[2] = z_;
		}
		
		Vector3f (const Vector3f& v)
		{
			p[0] = v[0];
			p[1] = v[1];
			p[2] = v[2];
		}
		
		
		operator const float* () const {
			return p;
		}
		operator float* () {
			return p;
		}
		
		float& operator[] (int i)
		{
			return p[i];
		}
		
		Vector3f operator+ ( Vector3d v )
		{
			return Vector3f ( p[0] + v[0], p[1] + v[1], p[2] + v[2] );
		}
		
		void operator= ( Vector3f v )
		{
			p[0] = v[0];
			p[1] = v[1];
			p[2] = v[2];
		}
		
		void operator+= ( Vector3f v )
		{
			p[0] += v[0]; p[1] += v[1]; p[2] += v[2];
		}
		
		void operator/= ( float v )
		{
			p[0] /= v; p[1] /= v; p[2] /= v;
		}
		
		Vector3f operator- ( Vector3f v )
		{
			return Vector3f ( p[0] - v[0], p[1] - v[1], p[2] - v[2] );
		}
		
		Vector3f operator* ( float v )
		{
			return Vector3f ( p[0]*v, p[1]*v, p[2]*v );
		}
		
		float Length ()
		{
			return sqrt ( p[0]*p[0] + p[1]*p[1] + p[2]*p[2] );
		}
		
		float SquaredLength ()
		{
			return  p[0]*p[0] + p[1]*p[1] + p[2]*p[2];
		}
	};



class Vector3i
	{
	public:
		int p[3];
		Vector3i () {
			p[0] = 0;
			p[1] = 0;
			p[2] = 0;
		}
		
		Vector3i ( int x_, int y_, int z_ )
		{
			p[0] = x_;
			p[1] = y_;
			p[2] = z_;
		}

		inline operator const int* () const {
			return p;
		}
		inline operator int* () {
			return p;
		}
		
	};

#endif